@@ -24,7 +24,8 @@ def getVersion():
         import subprocess
         try:
             version = subprocess.check_output("git describe --tags --dirty".split(" "),
-                cwd=root, universal_newlines=True, stderr=subprocess.DEVNULL)
+                cwd=root, universal_newlines=True, stderr=subprocess.DEVNULL,
+                errors='replace')
         except (subprocess.CalledProcessError, OSError):
             pass
 
@@ -61,7 +62,7 @@ def getVersion():
 
     if not version:
         # Last fallback. See PEP 440 and adjust accordingly.
-        version = "0.20+unknown"
+        version = "0.21+unknown"
 
     return version
 
