@@ -18,7 +18,7 @@ from bob.utils import removePath, isWindows
 from bob.errors import BuildError
 from bob.utils import summonMagic, hashFile
 from collections import OrderedDict, namedtuple
-from bob.tty import colorize
+from bob.tty import colorize, WARNING
 from shlex import quote
 
 # helper to get linux or windows (MSYS2 support) cwd
@@ -301,8 +301,7 @@ def qtProjectGenerator(package, argv, extra, bobRoot):
     for i in args.additional_includes:
         for e in parseArgumentLine(i):
             if os.path.exists(e):
-                for root, directories, filenames in os.walk(e):
-                    hList.append(os.path.join(e,root))
+                hList.append(e)
 
     # compose start includes
     for i in args.start_includes:
@@ -318,6 +317,7 @@ def qtProjectGenerator(package, argv, extra, bobRoot):
     id = None
     name = None
     kits = []
+    allKits = []
     try:
         if isWindows():
             profiles = xml.etree.ElementTree.parse(os.path.join(os.getenv('APPDATA'), "QtProject/qtcreator/profiles.xml")).getroot()
@@ -332,17 +332,24 @@ def qtProjectGenerator(package, argv, extra, bobRoot):
                         id = str(value.text)
                     if (value.attrib.get('key') == 'PE.Profile.Name'):
                         name = str(value.text)
-                    if (id is not None) and (name is not None) and (_kit.search(name)):
-                        kits.append([name, id])
-                        break
+                if not id or not name:
+                    continue
+                allKits.append(name)
+                if _kit.search(name):
+                    kits.append([name, id])
     except FileNotFoundError:
-        # if kits are generared using sdk tool they are stored somewhere else... (/usr/share...)
-        pass
+        print(colorize("Qt Creator settings could not be found! " \
+            "Make sure Qt Creator is installed and works properly.", WARNING))
 
     if (len(kits) == 0):
         if (args.kit is None):
-            raise BuildError("No kit found!",
-                help = "Run again with '--kit' and specify a kit or generate a Desktop kit, which is used by default.")
+            if allKits:
+                allKits = "The following kits were found: " + ", ".join(["'"+k+"'" for k in allKits])
+            else:
+                allKits = "No kits could be found!"
+            raise BuildError("No default kit found!",
+                help = "Run again with '--kit' to manually specify a kit. " \
+                       "See the bob-project manpage for more information.\n" + allKits)
         kitName = args.kit
         kitId = args.kit
     else:
