@@ -79,7 +79,8 @@ class InvocationMode(Enum):
     SHELL = 'shell'
 
 class Invoker:
-    def __init__(self, spec, preserveEnv, noLogFiles, showStdOut, showStdErr, trace, redirect):
+    def __init__(self, spec, preserveEnv, noLogFiles, showStdOut, showStdErr,
+                 trace, redirect, executor=None):
         self.__spec = spec
         self.__cwd = spec.workspaceWorkspacePath
         self.__preserveEnv = preserveEnv
@@ -95,6 +96,8 @@ class Invoker:
         self.__trace = trace
         self.__sandboxHelperPath = None
         self.__stdioBuffer = io.BytesIO() if redirect else None
+        self.__warnedDuplicates = { '' }
+        self.__executor = executor
 
         # Redirection is a bit complicated. We have to consider two levels: the
         # optional log file and the console.
@@ -171,7 +174,7 @@ class Invoker:
 
     async def __runCommand(self, args, cwd, stdout=None, stderr=None,
                            check=False, env=None, universal_newlines=True,
-                           specEnv=True, **kwargs):
+                           errors='replace', specEnv=True, **kwargs):
         cmd = " ".join(quote(a) for a in args)
         self.trace(cmd)
 
@@ -206,17 +209,22 @@ class Invoker:
 
         # Sanity check on Windows that there are no environment variables that
         # differ only in case. The Windows envrionment is used case insensitive
-        # by libc getenv() but the kernel passes the variables as-is.
+        # by libc getenv() but the kernel passes the variables as-is. We warn
+        # only about variables that are defined in the recipe. It's not our
+        # business if the inherited OS environment already has duplicates.
         if isWindows():
-            duplicates = []
-            old = ""
-            for i in sorted(env.keys(), key=str.upper):
-                if i.upper() == old.upper():
-                    duplicates.append((old, i))
-                old = i
-            if duplicates:
-                self.fail("Colliding environment variables:",
-                    ", ".join("{} ~= {}".format(i, j) for i,j in duplicates))
+            duplicates = set()
+            definedEnvVars = set(self.__spec.env.keys())
+            allEnvVars = set(env.keys()) | definedEnvVars
+            for i in definedEnvVars:
+                matchedVars = [ v for v in allEnvVars if v.upper() == i.upper() ]
+                if len(matchedVars) > 1:
+                    duplicates.add(" vs. ".join(sorted(matchedVars)))
+            duplicates = ", ".join(sorted(duplicates))
+            if duplicates not in self.__warnedDuplicates:
+                self.warn("Duplicate environment variables:", duplicates+"!",
+                    "It is unspecified which variant is used by the invoked processes.")
+                self.__warnedDuplicates.add(duplicates)
 
         loop = asyncio.get_event_loop()
         exitFuture = asyncio.Future()
@@ -241,7 +249,7 @@ class Invoker:
         if stdout == True:
             if universal_newlines:
                 stdoutStream.seek(0)
-                stdoutBuf = io.TextIOWrapper(stdoutStream).read()
+                stdoutBuf = io.TextIOWrapper(stdoutStream, errors=errors).read()
             else:
                 stdoutBuf = stdoutStream.getvalue()
         else:
@@ -250,7 +258,7 @@ class Invoker:
         if stderr == True:
             if universal_newlines:
                 stderrStream.seek(0)
-                stderrBuf = io.TextIOWrapper(stderrStream).read()
+                stderrBuf = io.TextIOWrapper(stderrStream, errors=errors).read()
             else:
                 stderrBuf = stderrStream.getvalue()
         else:
@@ -583,8 +591,12 @@ class Invoker:
 
     def getStdio(self):
         return self.__stdioBuffer.getvalue().decode(
-            locale.getpreferredencoding(), 'surrogateescape')
+            locale.getpreferredencoding(), 'replace')
 
     def setMakeParameters(self, fds, jobs):
         self.__makeFds = fds
         self.__makeJobs = jobs
+
+    async def runInExecutor(self, func, *args):
+        loop = asyncio.get_event_loop()
+        return await loop.run_in_executor(self.__executor, func, *args)
