@@ -413,12 +413,13 @@ class PwshResolver(IncludeResolver):
     def _includeFile(self, content):
         var = "$_{}{}".format(self.varBase, self.count)
         self.count += 1
-        self.prolog.extend([
-            "{VAR} = (New-TemporaryFile).FullName".format(VAR=var),
-            "$_BOB_TMP_CLEANUP += {VAR}".format(VAR=var),
-            "[Convert]::FromBase64String(@'"])
+        self.prolog.append(dedent("""\
+            {VAR} = (New-TemporaryFile).FullName
+            $_BOB_TMP_CLEANUP += {VAR}
+            [io.file]::WriteAllBytes({VAR}, [Convert]::FromBase64String(@'"""
+                .format(VAR=var)))
         self.prolog.extend(sliceString(b64encode(content).decode("ascii"), 76))
-        self.prolog.append("'@) | Set-Content {VAR} -AsByteStream".format(VAR=var))
+        self.prolog.append("'@))")
         return var
 
     def _includeLiteral(self, content):
@@ -661,9 +662,13 @@ class StepSpec:
                 ],
             }
 
-            # special handling to mount all previous steps of current package
+            # Special handling to mount all previous steps of current package.
+            # It is defined that the checkout and build step are visible in the
+            # sandbox for a given package step. We must stop at checkout steps
+            # because they might have dependencies due to the 'checkoutDep'
+            # flag.
             extra = step
-            while extra.isValid() and len(extra.getArguments()) > 0:
+            while extra.isValid() and not extra.isCheckoutStep() and len(extra.getArguments()) > 0:
                 extra = extra.getArguments()[0]
                 if extra.isValid():
                     s['depMounts'].append((extra.getStoragePath(), extra.getExecPath(step)))
