@@ -11,14 +11,19 @@ import subprocess
 
 class CvsScm(Scm):
 
-    SCHEMA = schema.Schema({
+    DEFAULTS = {
+        schema.Optional('dir') : str
+    }
+
+    __SCHEMA = {
         'scm' : 'cvs',
         'cvsroot' : str,
         'module' : str,
-        schema.Optional('dir') : str,
         schema.Optional('if') : str,
         schema.Optional('rev') : str
-    })
+    }
+
+    SCHEMA = schema.Schema({**__SCHEMA, **DEFAULTS})
 
     # Checkout using CVS
     # - mandatory parameters: cvsroot, module
@@ -132,6 +137,7 @@ class CvsScm(Scm):
                 output = subprocess.check_output(commandLine,
                                                  cwd=workDir,
                                                  universal_newlines=True,
+                                                 errors='replace',
                                                  env=environment,
                                                  stderr=subprocess.DEVNULL,
                                                  stdin=subprocess.DEVNULL)
