@@ -30,23 +30,35 @@ def dirIsEmpty(p):
 
 class GitScm(Scm):
 
-    SCHEMA = schema.Schema({
+    DEFAULTS = {
+        schema.Optional('branch') : str,
+        schema.Optional('sslVerify') : bool,
+        schema.Optional('singleBranch') : bool,
+        schema.Optional('submodules') : schema.Or(bool, [str]),
+        schema.Optional('recurseSubmodules') : bool,
+        schema.Optional('shallowSubmodules') : bool,
+        schema.Optional('shallow') : schema.Or(int, str),
+        schema.Optional('dir') : str,
+        schema.Optional('references') :
+            schema.Schema([schema.Or(str, {
+                schema.Optional('url') : str,
+                'repo'    : str,
+                schema.Optional('optional') : bool,
+            })]),
+        schema.Optional('dissociate') : bool,
+    }
+
+    __SCHEMA = {
         'scm' : 'git',
         'url' : str,
-        schema.Optional('dir') : str,
         schema.Optional('if') : schema.Or(str, IfExpression),
-        schema.Optional('branch') : str,
         schema.Optional('tag') : str,
         schema.Optional('commit') : str,
         schema.Optional('rev') : str,
         schema.Optional(schema.Regex('^remote-.*')) : str,
-        schema.Optional('sslVerify') : bool,
-        schema.Optional('singleBranch') : bool,
-        schema.Optional('shallow') : schema.Or(int, str),
-        schema.Optional('submodules') : schema.Or(bool, [str]),
-        schema.Optional('recurseSubmodules') : bool,
-        schema.Optional('shallowSubmodules') : bool,
-    })
+    }
+
+    SCHEMA = schema.Schema({**__SCHEMA, **DEFAULTS})
     REMOTE_PREFIX = "remote-"
 
     def __init__(self, spec, overrides=[], secureSSL=None, stripUser=None):
@@ -91,6 +103,26 @@ class GitScm(Scm):
         self.__recurseSubmodules = spec.get('recurseSubmodules', False)
         self.__shallowSubmodules = spec.get('shallowSubmodules', True)
         self.__stripUser = stripUser
+        self.__references = spec.get('references')
+        self.__dissociate = spec.get('dissociate', False)
+        self.__resolvedReferences = []
+
+        def __resolveReferences(self, alt):
+            # if the reference is a string it's used as optional reference
+            if isinstance(alt, str):
+                return (alt, True)
+            else:
+                try:
+                    sub = re.sub(alt.get('url', '.*'), alt['repo'], self.__url)
+                    if sub == self.__url:
+                        raise ParseError("`url` pattern '" +
+                            alt.get('url','.*') + " did not apply to: " + self.__url)
+                    return (sub, alt.get('optional', True))
+                except re.error:
+                    raise ParseError("Invalid `url` pattern: " + alt.get('url', '.*'))
+        if self.__references is not None:
+            self.__resolvedReferences = [ ref for ref in [__resolveReferences(self, a) for a in
+                self.__references]]
 
     def getProperties(self, isJenkins):
         properties = super().getProperties(isJenkins)
@@ -111,15 +143,34 @@ class GitScm(Scm):
             'submodules' : self.__submodules,
             'recurseSubmodules' : self.__recurseSubmodules,
             'shallowSubmodules' : self.__shallowSubmodules,
+            'references' : self.__references,
+            'dissociate' : self.__dissociate,
         })
         for key, val in self.__remotes.items():
             properties.update({GitScm.REMOTE_PREFIX+key : val})
         return properties
 
     async def invoke(self, invoker, switch=False):
+        alternatesFile = invoker.joinPath(self.__dir, ".git/objects/info/alternates")
+
         # make sure the git directory exists
         if not os.path.isdir(invoker.joinPath(self.__dir, ".git")):
             await invoker.checkCommand(["git", "init", self.__dir])
+            # setup local reference repo by writing the alternates file
+            if self.__resolvedReferences:
+                with open(alternatesFile, "w") as a:
+                    for (ref, optional) in self.__resolvedReferences:
+                        refPath=None
+                        if os.path.isdir(os.path.join(ref, ".git", "objects")):
+                            refPath = ref + "/.git/objects\n"
+                        elif os.path.isdir(os.path.join(ref, "objects")):
+                            refPath = ref + "/objects\n"
+                        elif not optional:
+                            raise BuildError("Unable to use reference for '" + self.__url + "'. '" +
+                                ref + "' is not existing'.")
+                        if refPath:
+                            invoker.info("Add git reference: " + refPath)
+                            a.write(refPath)
 
         # Shallow implies singleBranch
         if self.__singleBranch is None:
@@ -176,12 +227,32 @@ class GitScm(Scm):
         else:
             await self.__checkoutBranch(invoker, fetchCmd, switch)
 
+        if os.path.exists(alternatesFile) and self.__dissociate:
+            await invoker.checkCommand(["git", "repack", "-a"], cwd=self.__dir)
+            os.unlink(alternatesFile)
+
     async def __checkoutTag(self, invoker, fetchCmd, switch):
         # checkout only if HEAD is invalid
         head = await invoker.callCommand(["git", "rev-parse", "--verify", "-q", "HEAD"],
             stdout=False, cwd=self.__dir)
         if head or switch:
             await invoker.checkCommand(fetchCmd, cwd=self.__dir)
+            if self.__commit and (self.__shallow is not None):
+                # Shallow clones might not fetch the requested commit if the
+                # depth is too small. The problem is that we cannot blindly
+                # always request the commit explicitly because this is rejected
+                # by git-upload-pack by default. Instead check here that we got
+                # the commit and only if this is not the case we fetch a 2nd
+                # time with the explicit commit.
+                haveCommit = await invoker.callCommand(["git", "cat-file", "-e",
+                    self.__commit], cwd=self.__dir) == 0
+                if not haveCommit:
+                    ok = await invoker.callCommand(fetchCmd + [self.__commit],
+                        cwd=self.__dir)
+                    if ok != 0:
+                        invoker.fail("Plain git-fetch in", self.__dir,
+                            "did not download the requested commit and the explicit fetch failed!",
+                            returncode=ok)
             await invoker.checkCommand(["git", "checkout", "-q", "--no-recurse-submodules",
                 self.__commit if self.__commit else "tags/"+self.__tag], cwd=self.__dir)
             # FIXME: will not be called again if interrupted!
@@ -198,18 +269,20 @@ class GitScm(Scm):
         elif switch:
             # We're switching the ref. There we will actively change the branch which
             # is normally forbidden.
-            assert not self.__submodules
             if await invoker.callCommand(["git", "show-ref", "-q", "--verify",
                                           "refs/heads/" + self.__branch]):
                 # Branch does not exist. Create and checkout.
                 await invoker.checkCommand(["git", "checkout", "--no-recurse-submodules",
                     "-b", self.__branch, "remotes/origin/"+self.__branch], cwd=self.__dir)
+                await self.__checkoutSubmodules(invoker)
             else:
                 # Branch exists already. Checkout and fast forward...
                 await invoker.checkCommand(["git", "checkout", "--no-recurse-submodules",
                     self.__branch], cwd=self.__dir)
+                preUpdate = await self.__updateSubmodulesPre(invoker)
                 await invoker.checkCommand(["git", "-c", "submodule.recurse=0", "merge",
                     "--ff-only", "refs/remotes/origin/"+self.__branch], cwd=self.__dir)
+                await self.__updateSubmodulesPost(invoker, preUpdate)
         elif (await invoker.checkOutputCommand(["git", "rev-parse", "--abbrev-ref", "HEAD"], cwd=self.__dir)) == self.__branch:
             # pull only if on original branch
             preUpdate = await self.__updateSubmodulesPre(invoker)
@@ -226,6 +299,7 @@ class GitScm(Scm):
             args += ["--depth", "1"]
         if self.__recurseSubmodules:
             args += ["--recursive"]
+
         if isinstance(self.__submodules, list):
             args.append("--")
             args.extend(self.__submodules)
@@ -513,7 +587,7 @@ class GitScm(Scm):
         cwd = os.path.join(workspacePath, self.__dir)
         try:
             output = subprocess.check_output(cmdLine, cwd=cwd,
-                universal_newlines=True, stderr=subprocess.DEVNULL)
+                universal_newlines=True, stderr=subprocess.DEVNULL, errors='replace')
         except subprocess.CalledProcessError as e:
             if check:
                 raise BuildError("git error:\n Directory: '{}'\n Command: '{}'\n'{}'".format(
@@ -700,7 +774,7 @@ class GitScm(Scm):
             cmdLine = ['git', 'ls-remote', self.__url] + refs
             try:
                 stdout = await check_output(cmdLine, stderr=subprocess.DEVNULL,
-                    universal_newlines=True)
+                    universal_newlines=True, errors='replace')
                 output = stdout.strip()
             except subprocess.CalledProcessError as e:
                 a.fail("exit {}".format(e.returncode), WARNING)
@@ -750,7 +824,7 @@ class GitScm(Scm):
     def processLiveBuildIdSpec(dir):
         try:
             return subprocess.check_output(["git", "rev-parse", "HEAD"],
-                cwd=dir, universal_newlines=True).strip()
+                cwd=dir, universal_newlines=True, errors='replace').strip()
         except subprocess.CalledProcessError as e:
             raise BuildError("Git audit failed: " + str(e))
         except OSError as e:
@@ -776,15 +850,15 @@ class GitAudit(ScmAudit):
         dir = os.path.join(workspace, dir)
         try:
             remotes = (await check_output(["git", "remote", "-v"],
-                cwd=dir, universal_newlines=True)).split("\n")
+                cwd=dir, universal_newlines=True, errors='replace')).split("\n")
             remotes = (r[:-8].split("\t") for r in remotes if r.endswith("(fetch)"))
             self.__remotes = { remote:url for (remote,url) in remotes }
 
             self.__commit = (await check_output(["git", "rev-parse", "HEAD"],
-                cwd=dir, universal_newlines=True)).strip()
+                cwd=dir, universal_newlines=True, errors='replace')).strip()
             self.__description = (await check_output(
                 ["git", "describe", "--always", "--dirty=-dirty"],
-                cwd=dir, universal_newlines=True)).strip()
+                cwd=dir, universal_newlines=True, errors='replace')).strip()
             subDirty = await self.__scanSubmodules(dir, self.__submodules)
             self.__dirty = subDirty or self.__description.endswith("-dirty")
         except subprocess.CalledProcessError as e:
@@ -801,7 +875,7 @@ class GitAudit(ScmAudit):
         try:
             allPaths = await check_output(["git", "-C", base, "config", "-f",
                 ".gitmodules", "-z", "--get-regexp", "path"], cwd=dir,
-                universal_newlines=True)
+                universal_newlines=True, errors='replace')
         except subprocess.CalledProcessError:
             allPaths = "" # No key found in file. Probably empty
         allPaths = [ p.split("\n")[1] for p in allPaths.split("\0") if p ]
@@ -810,7 +884,7 @@ class GitAudit(ScmAudit):
 
         # Fetch the respecive commits as per git ls-tree
         allPaths = await check_output(["git", "-C", base, "ls-tree", "-z",
-            "HEAD"] + allPaths, cwd=dir, universal_newlines=True)
+            "HEAD"] + allPaths, cwd=dir, universal_newlines=True, errors='replace')
         allPaths = {
             path : attribs.split(' ')[2]
                 for attribs, path
@@ -844,7 +918,7 @@ class GitAudit(ScmAudit):
                 return True
 
             realCommit = (await check_output(["git", "-C", subPath, "rev-parse", "HEAD"],
-                cwd=dir, universal_newlines=True)).strip()
+                cwd=dir, universal_newlines=True, errors='replace')).strip()
             if commit != realCommit:
                 return True # different commit checked out
 
