@@ -6,10 +6,12 @@
 from .. import BOB_VERSION
 from ..errors import BuildError, ParseError
 from ..stringparser import IfExpression
-from ..utils import asHexStr, hashFile, isWindows, removeUserFromUrl
+from ..utils import asHexStr, hashFile, isWindows, removeUserFromUrl, sslNoVerifyContext, \
+        replacePath
 from .scm import Scm, ScmAudit
+from http.client import HTTPException
 import asyncio
-import concurrent
+import concurrent.futures.process
 import contextlib
 import hashlib
 import os, os.path
@@ -115,18 +117,24 @@ def parseUrl(url):
 
 class UrlScm(Scm):
 
-    SCHEMA = schema.Schema({
+    DEFAULTS = {
+        schema.Optional('extract') : schema.Or(bool, str),
+        schema.Optional('fileName') : str,
+        schema.Optional('stripComponents') : int,
+        schema.Optional('sslVerify') : bool,
+    }
+
+    __SCHEMA = {
         'scm' : 'url',
         'url' : str,
         schema.Optional('dir') : str,
         schema.Optional('if') : schema.Or(str, IfExpression),
         schema.Optional('digestSHA1') : str,
         schema.Optional('digestSHA256') : str,
-        schema.Optional('extract') : schema.Or(bool, str),
-        schema.Optional('fileName') : str,
-        schema.Optional('stripComponents') : int,
-        schema.Optional('sslVerify') : bool,
-    })
+        schema.Optional('digestSHA512') : str,
+    }
+
+    SCHEMA = schema.Schema({**__SCHEMA, **DEFAULTS})
 
     EXTENSIONS = [
         (".tar.gz",    "tar"),
@@ -134,6 +142,7 @@ class UrlScm(Scm):
         (".tar.bz2",   "tar"),
         (".tar.bzip2", "tar"),
         (".tgz",       "tar"),
+        (".txz",       "tar"),
         (".tar",       "tar"),
         (".gz",        "gzip"),
         (".xz",        "xz"),
@@ -168,12 +177,17 @@ class UrlScm(Scm):
         if self.__digestSha1:
             # validate digest
             if re.match("^[0-9a-f]{40}$", self.__digestSha1) is None:
-                raise ParseError("Invalid SHA1 digest: " + str(self.__digestSha1))
+                raise ParseError("Invalid SHA1 digest: '" + str(self.__digestSha1) + "'")
         self.__digestSha256 = spec.get("digestSHA256")
         if self.__digestSha256:
             # validate digest
             if re.match("^[0-9a-f]{64}$", self.__digestSha256) is None:
-                raise ParseError("Invalid SHA256 digest: " + str(self.__digestSha256))
+                raise ParseError("Invalid SHA256 digest: '" + str(self.__digestSha256) + "'")
+        self.__digestSha512 = spec.get("digestSHA512")
+        if self.__digestSha512:
+            # validate digest
+            if re.match("^[0-9a-f]{128}$", self.__digestSha512) is None:
+                raise ParseError("Invalid SHA512 digest: '" + str(self.__digestSha512) + "'")
         self.__dir = spec.get("dir", ".")
         self.__fn = spec.get("fileName")
         if not self.__fn:
@@ -196,6 +210,7 @@ class UrlScm(Scm):
             'url' : self.__url,
             'digestSHA1' : self.__digestSha1,
             'digestSHA256' : self.__digestSha256,
+            'digestSHA512' : self.__digestSha512,
             'dir' : self.__dir,
             'fileName' : self.__fn,
             'extract' : self.__extract,
@@ -207,7 +222,7 @@ class UrlScm(Scm):
     def _download(self, destination):
         headers = {}
         headers["User-Agent"] = "BobBuildTool/{}".format(BOB_VERSION)
-        context = None if self.__sslVerify else ssl.SSLContext(ssl.PROTOCOL_SSLv23)
+        context = None if self.__sslVerify else sslNoVerifyContext()
         if os.path.isfile(destination) and self.__url.startswith("http"):
             # Try to avoid download if possible
             headers["If-Modified-Since"] = time2HTTPDate(os.stat(destination).st_mtime)
@@ -238,12 +253,14 @@ class UrlScm(Scm):
                 # Atomically move file to destination. Set explicit mode to
                 # retain Bob 0.15 behaviour.
                 os.chmod(tmpFileName, stat.S_IREAD|stat.S_IWRITE)
-                os.replace(tmpFileName, destination)
+                replacePath(tmpFileName, destination)
                 tmpFileName = None
 
         except urllib.error.HTTPError as e:
             if e.code != 304:
                 return "HTTP error {}: {}".format(e.code, e.reason)
+        except HTTPException as e:
+            return "HTTP error: " + str(e)
         finally:
             if tmpFileName is not None:
                 os.remove(tmpFileName)
@@ -261,7 +278,7 @@ class UrlScm(Scm):
 
         # Adding, changing or removing hash sums is ok as long as the url stays
         # the same.
-        return diff.issubset({"digestSHA1", "digestSHA256"})
+        return diff.issubset({"digestSHA1", "digestSHA256", "digestSHA512"})
 
     async def switch(self, invoker, oldSpec):
         # The real work is done in invoke() below. It will fail if the file
@@ -292,9 +309,8 @@ class UrlScm(Scm):
                     shutil.copy(url.path, destination)
             elif url.scheme in ["http", "https", "ftp"]:
                 invoker.trace("<wget>", self.__url, ">", workspaceFile)
-                loop = asyncio.get_event_loop()
                 try:
-                    err = await loop.run_in_executor(None, UrlScm._download, self, destination)
+                    err = await invoker.runInExecutor(UrlScm._download, self, destination)
                     if err:
                         invoker.fail(err)
                 except (concurrent.futures.CancelledError, concurrent.futures.process.BrokenProcessPool):
@@ -314,6 +330,11 @@ class UrlScm(Scm):
             d = hashFile(destination, hashlib.sha256).hex()
             if d != self.__digestSha256:
                 invoker.fail("SHA256 digest did not match! expected:", self.__digestSha256, "got:", d)
+        if self.__digestSha512:
+            invoker.trace("<sha512sum>", workspaceFile)
+            d = hashFile(destination, hashlib.sha512).hex()
+            if d != self.__digestSha512:
+                invoker.fail("SHA512 digest did not match! expected:", self.__digestSha512, "got:", d)
 
         # Run optional extractors
         extractors = self.__getExtractors()
@@ -341,16 +362,18 @@ class UrlScm(Scm):
             filt = removeUserFromUrl
         else:
             filt = lambda x: x
-        return ( self.__digestSha256 if self.__digestSha256
-                 else (self.__digestSha1 if self.__digestSha1 else filt(self.__url))
-                    ) + " " + os.path.join(self.__dir, self.__fn) + " " + str(self.__extract) + \
-                    ( " s{}".format(self.__strip) if self.__strip > 0 else "" )
+        return ( self.__digestSha512 or self.__digestSha256 or 
+                 self.__digestSha1 or filt(self.__url)
+               ) + " " + os.path.join(self.__dir, self.__fn) + " " + str(self.__extract) + \
+               ( " s{}".format(self.__strip) if self.__strip > 0 else "" )
 
     def getDirectory(self):
         return self.__dir if self.__tidy else os.path.join(self.__dir, self.__fn)
 
     def isDeterministic(self):
-        return (self.__digestSha1 is not None) or (self.__digestSha256 is not None)
+        return (self.__digestSha1 is not None) or \
+               (self.__digestSha256 is not None) or \
+               (self.__digestSha512 is not None)
 
     def getAuditSpec(self):
         return ("url", os.path.join(self.__dir, self.__fn),
@@ -363,7 +386,9 @@ class UrlScm(Scm):
         return self.calcLiveBuildId(None)
 
     def calcLiveBuildId(self, workspacePath):
-        if self.__digestSha256:
+        if self.__digestSha512:
+            return bytes.fromhex(self.__digestSha512)
+        elif self.__digestSha256:
             return bytes.fromhex(self.__digestSha256)
         elif self.__digestSha1:
             return bytes.fromhex(self.__digestSha1)
@@ -371,7 +396,9 @@ class UrlScm(Scm):
             return None
 
     def getLiveBuildIdSpec(self, workspacePath):
-        if self.__digestSha256:
+        if self.__digestSha512:
+            return "=" + self.__digestSha512
+        elif self.__digestSha256:
             return "=" + self.__digestSha256
         elif self.__digestSha1:
             return "=" + self.__digestSha1
