@@ -211,10 +211,25 @@ if os.name == 'posix':
 else:
     __isWindows = True
 
+def _replacePathWin32(src, dst):
+    # Workaround for spurious PermissionError's on Windows.
+    i = 0
+    while True:
+        try:
+            os.replace(src, dst)
+            break
+        except PermissionError:
+            if i >= 10: raise
+            import time
+            time.sleep(0.1 * i)
+            i += 1
+
 if __isWindows:
     INVALID_CHAR_TRANS = str.maketrans(':*?<>"|', '_______')
+    replacePath = _replacePathWin32
 else:
     INVALID_CHAR_TRANS = str.maketrans('', '')
+    replacePath = os.replace
 
 
 __canSymlink = None
@@ -352,7 +367,7 @@ class DirHasher:
                     self.__inFile.close()
                 if self.__outFile:
                     self.__outFile.close()
-                    os.replace(self.__outFile.name, self.__cachePath)
+                    replacePath(self.__outFile.name, self.__cachePath)
             except OSError as e:
                 raise BuildError("Error closing hash cache: " + str(e))
 
@@ -536,7 +551,7 @@ def hashPath(path, index=None, ignoreDirs=None):
 
 def binStat(path):
     st = os.stat(path)
-    return struct.pack('=qqLqLQ', st.st_ctime_ns, st.st_mtime_ns,
+    return struct.pack('=qqLQLQ', st.st_ctime_ns, st.st_mtime_ns,
                        st.st_dev, st.st_ino, st.st_mode, st.st_size)
 
 
@@ -631,17 +646,22 @@ def infixBinaryOp(handler, *args, **kwargs):
 def dummy():
     pass
 
-class EventLoopWrapper:
-    def __init__(self):
-        import asyncio
+__startMethodSet = False
+def __setStartMethod(method):
+    global __startMethodSet
+    if not __startMethodSet:
         import multiprocessing
-        import signal
-        import concurrent.futures
+        multiprocessing.set_start_method(method)
+        __startMethodSet = True
 
+def getProcessPoolExecutor():
+    import multiprocessing
+    import signal
+    import concurrent.futures
+
+    try:
         if sys.platform == 'win32':
-            loop = asyncio.ProactorEventLoop()
-            asyncio.set_event_loop(loop)
-            multiprocessing.set_start_method('spawn')
+            __setStartMethod('spawn')
             executor = concurrent.futures.ProcessPoolExecutor()
         else:
             # The ProcessPoolExecutor is a barely usable for our interactive use
@@ -651,31 +671,67 @@ class EventLoopWrapper:
             # have to ignore BrokenProcessPool errors as we will likely hit them.
             # To "prime" the process pool a dummy workload must be executed because
             # the processes are spawned lazily.
-            loop = asyncio.get_event_loop()
             origSigInt = signal.getsignal(signal.SIGINT)
             signal.signal(signal.SIGINT, signal.SIG_IGN)
-            # fork early before process gets big
-            if isWindows():
-                multiprocessing.set_start_method('fork')
+
+            method = 'fork' if isWindows() else 'forkserver'
+
+            # Hack to get coverage data with the multiprocessing module. Up to
+            # date coverage.py cannot trace cross fork/exec. Taken and adapted
+            # from https://gist.github.com/andreycizov/ee59806a3ac6955c127e511c5e84d2b6
+            if os.environ.get('ENABLE_COVERAGE_HACK') and (sys.version_info >= (3, 7)):
+                ctx = multiprocessing.get_context(method)
+                from coverage import Coverage
+                from multiprocessing.context import Process
+                class CoverageProcess(Process):
+                    def run(self):
+                        cov = Coverage(data_suffix=True)
+                        cov._warn_no_data = True
+                        cov._warn_unimported_source = True
+                        cov.start()
+                        try:
+                            super().run()
+                        finally:
+                            cov.stop()
+                            cov.save()
+                ctx.Process = CoverageProcess
+                executor = concurrent.futures.ProcessPoolExecutor(mp_context=ctx)
             else:
-                multiprocessing.set_start_method('forkserver')
-            executor = concurrent.futures.ProcessPoolExecutor()
+                __setStartMethod(method)
+                executor = concurrent.futures.ProcessPoolExecutor()
+
+            # fork early before process gets big
             executor.submit(dummy).result()
             signal.signal(signal.SIGINT, origSigInt)
-        loop.set_default_executor(executor)
+    except OSError as e:
+        raise BuildError("Error spawning process pool: " + str(e))
 
-        self.__loop = loop
-        self.__executor = executor
+    return executor
+
+class EventLoopWrapper:
+    def __init__(self):
+        import asyncio
+
+        if sys.platform == 'win32':
+            self.__loop = asyncio.ProactorEventLoop()
+        else:
+            self.__loop = asyncio.new_event_loop()
+
+        self.__executor = getProcessPoolExecutor()
 
     def __enter__(self):
-        return self.__loop
+        import asyncio
+        asyncio.set_event_loop(self.__loop)
+        return (self.__loop, self.__executor)
 
     def __exit__(self, exc_type, exc_value, traceback):
+        import asyncio
         self.__executor.shutdown()
         self.__loop.close()
+        asyncio.set_event_loop(None)
 
 
-async def run(args, universal_newlines=False, check=False, shell=False, **kwargs):
+async def run(args, universal_newlines=False, errors=None, check=False, shell=False, **kwargs):
     """Provide the subprocess.run() function as asyncio corouting.
 
     This takes care of the missing 'universal_newlines' and 'check' options.
@@ -694,9 +750,9 @@ async def run(args, universal_newlines=False, check=False, shell=False, **kwargs
     stdout, stderr = await proc.communicate()
 
     if universal_newlines and (stdout is not None):
-        stdout = io.TextIOWrapper(io.BytesIO(stdout)).read()
+        stdout = io.TextIOWrapper(io.BytesIO(stdout), errors=errors).read()
     if universal_newlines and (stderr is not None):
-        stderr = io.TextIOWrapper(io.BytesIO(stderr)).read()
+        stderr = io.TextIOWrapper(io.BytesIO(stderr), errors=errors).read()
 
     if check and (proc.returncode != 0):
         raise subprocess.CalledProcessError(proc.returncode, args,
@@ -709,3 +765,26 @@ async def check_output(args, **kwargs):
     """The subprocess.check_output() call as coroutine."""
     import subprocess
     return (await run(args, check=True, stdout=subprocess.PIPE, **kwargs)).stdout
+
+def runInEventLoop(coro):
+    """Backwards compatibility stub for asyncio.run()"""
+    import asyncio
+
+    if sys.version_info.minor >= 7:
+        return asyncio.run(coro)
+
+    loop = asyncio.new_event_loop()
+    try:
+        asyncio.set_event_loop(loop)
+        return loop.run_until_complete(coro)
+    finally:
+        asyncio.set_event_loop(None)
+        loop.close()
+
+def sslNoVerifyContext():
+    """Generate a SSL context that does not validate certificates."""
+    import ssl
+    context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
+    context.check_hostname = False
+    context.verify_mode = ssl.CERT_NONE
+    return context
